<?php

namespace Drp\SimpleCms\Generators\Commands;

use Drp\SimpleCms\Generators\MigrationCreator;
use Drp\SimpleCms\Generators\ViewCreator;
use Illuminate\Database\Console\Migrations\BaseCommand;
use Illuminate\Support\Composer;
use Symfony\Component\Console\Input\InputArgument;

class ContentMakeCommand extends BaseCommand
{
    /**
     * @var string
     */
    protected $name = 'make:content';

    /**
     * @var string
     */
    protected $description = 'Create a new content class and apply schema at the same time';

    /**
     * @var MigrationCreator
     */
    private $migrationCreator;

    /**
     * @var Composer
     */
    private $composer;

    /**
     * @var ViewCreator
     */
    private $viewCreator;

    public function __construct(
        MigrationCreator $migrationCreator,
        ViewCreator $viewCreator,
        Composer $composer
    ) {
        $this->migrationCreator = $migrationCreator;
        $this->composer = $composer;
        $this->viewCreator = $viewCreator;
        parent::__construct();
    }

    /**
     * Helper to allow both older versions of Laravel fire() and newer handle()
     *
     * @return mixed
     */
    public function fire()
    {
        return $this->handle();
    }

    /**
     * @return mixed
     */
    public function handle()
    {
        $this->name = $this->argument('name');

        $this->makeMigration($this->getMigrationName());

        $this->call('make:content-model', [
            'name' => $this->name,
        ]);

        $this->viewCreator->create($this->getViewPath());
    }

    private function makeMigration(): void
    {
        $file = pathinfo($this->migrationCreator->create(
            $this->getMigrationName(),
            $this->getMigrationPath(),
            $this->getTableName()
        ), PATHINFO_FILENAME);

        $this->line("<info>Created Migration:</info> {$file}");

        $this->composer->dumpAutoloads();
    }

    protected function getMigrationName(): string
    {
        return 'create_' . strtolower(str_plural(snake_case($this->name))) . '_table';
    }

    protected function getTableName(): string
    {
        return str_plural(snake_case($this->name));
    }

    protected function getArguments(): array
    {
        return [
            ['name', InputArgument::REQUIRED, 'The name of the content'],
        ];
    }

    protected function getMigrationPath(): string
    {
        return $this->laravel->databasePath() . DIRECTORY_SEPARATOR . 'migrations';
    }

    protected function getViewPath(): string
    {
        return $this->laravel->resourcePath() . DIRECTORY_SEPARATOR . implode(
            DIRECTORY_SEPARATOR,
            [
                'views',
                'admin',
                'content',
                lcfirst(studly_case(($this->name))),
            ]
        );
    }
}
